@@ -38,26 +38,43 @@ static INLINE int PyObject_allowed_in_string_attribute(PyObject* o) {
   return o == Py_None || (o != 0 && PyBaseString_Check(o));
 }
 
+static INLINE void igraphmodule_i_attribute_struct_invalidate_vertex_name_index(
+    igraphmodule_i_attribute_struct *attrs
+);
+
 int igraphmodule_i_attribute_struct_init(igraphmodule_i_attribute_struct *attrs) {
-  int i;
-  for (i=0; i<3; i++) {
+  int i, j;
+  
+  for (i = 0; i < 3; i++) {
     attrs->attrs[i] = PyDict_New();
-    if (PyErr_Occurred())
+    if (PyErr_Occurred()) {
+      for (j = 0; j < i; j++) {
+        RC_DEALLOC("dict", attrs->attrs[j]);
+        Py_DECREF(attrs->attrs[j]);
+        attrs->attrs[j] = NULL;
+      }
       return 1;
+    }
+
     RC_ALLOC("dict", attrs->attrs[i]);
   }
-  attrs->vertex_name_index = 0;
+
+  attrs->vertex_name_index = NULL;
+
   return 0;
 }
 
 void igraphmodule_i_attribute_struct_destroy(igraphmodule_i_attribute_struct *attrs) {
   int i;
-  for (i=0; i<3; i++) {
+
+  for (i = 0; i < 3; i++) {
     if (attrs->attrs[i]) {
       RC_DEALLOC("dict", attrs->attrs[i]);
       Py_DECREF(attrs->attrs[i]);
+      attrs->attrs[i] = NULL;
     }
   }
+
   if (attrs->vertex_name_index) {
     RC_DEALLOC("dict", attrs->vertex_name_index);
     Py_DECREF(attrs->vertex_name_index);
@@ -68,32 +85,37 @@ int igraphmodule_i_attribute_struct_index_vertex_names(
     igraphmodule_i_attribute_struct *attrs, igraph_bool_t force) {
   Py_ssize_t n = 0;
   PyObject *name_list, *key, *value;
+  igraph_bool_t success = false;
 
-  if (attrs->vertex_name_index && !force)
+  if (attrs->vertex_name_index && !force) {
     return 0;
+  }
 
   if (attrs->vertex_name_index == 0) {
     attrs->vertex_name_index = PyDict_New();
     if (attrs->vertex_name_index == 0) {
-      return 1;
+      goto cleanup;
     }
-  } else
-    PyDict_Clear(attrs->vertex_name_index);
+  }
 
-  name_list = PyDict_GetItemString(attrs->attrs[1], "name");
-  if (name_list == 0)
-    return 0;    /* no name attribute */
+  PyDict_Clear(attrs->vertex_name_index);
+
+  name_list = PyDict_GetItemString(attrs->attrs[ATTRHASH_IDX_VERTEX], "name");
+  if (name_list == 0) {
+    success = true;
+    goto cleanup;
+  }
 
   n = PyList_Size(name_list) - 1;
   while (n >= 0) {
     key = PyList_GetItem(name_list, n);      /* we don't own a reference to key */
     if (key == 0) {
-      return 1;
+      goto cleanup;
     }
 
     value = PyLong_FromLong(n);              /* we do own a reference to value */
     if (value == 0) {
-      return 1;
+      goto cleanup;
     }
 
     if (PyDict_SetItem(attrs->vertex_name_index, key, value)) {
@@ -108,7 +130,10 @@ int igraphmodule_i_attribute_struct_index_vertex_names(
         );
       }
 
-      return 1;
+      /* Drop reference to value because we still own it */
+      Py_DECREF(value);
+
+      goto cleanup;
     }
 
     /* PyDict_SetItem did an INCREF for both the key and a value, therefore we
@@ -118,16 +143,24 @@ int igraphmodule_i_attribute_struct_index_vertex_names(
     n--;
   }
 
-  return 0;
+  success = true;
+
+cleanup:
+  if (!success) {
+    igraphmodule_i_attribute_struct_invalidate_vertex_name_index(attrs);
+  }
+
+  return success ? 0 : 1;
 }
 
-void igraphmodule_i_attribute_struct_invalidate_vertex_name_index(
-    igraphmodule_i_attribute_struct *attrs) {
-  if (attrs->vertex_name_index == 0)
-    return;
-
-  Py_DECREF(attrs->vertex_name_index);
-  attrs->vertex_name_index = 0;
+static void igraphmodule_i_attribute_struct_invalidate_vertex_name_index(
+    igraphmodule_i_attribute_struct *attrs
+) {
+  if (attrs->vertex_name_index) {
+    RC_DEALLOC("dict", attrs->vertex_name_index);
+    Py_DECREF(attrs->vertex_name_index);
+    attrs->vertex_name_index = NULL;
+  }
 }
 
 void igraphmodule_invalidate_vertex_name_index(igraph_t *graph) {
@@ -224,15 +257,19 @@ igraph_bool_t igraphmodule_has_edge_attribute(const igraph_t *graph, const char*
  *           attribute exists already (no exception set). The returned
  *           reference is borrowed.
  */
-PyObject* igraphmodule_create_edge_attribute(const igraph_t* graph,
+PyObject* igraphmodule_i_create_edge_attribute(const igraph_t* graph,
     const char* name) {
   PyObject *dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE];
   PyObject *values;
   Py_ssize_t i, n;
 
-  if (dict == 0) {
+  if (dict == NULL) {
     dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE] = PyDict_New();
+    if (dict == NULL) {
+      return NULL;
+    }
   }
+
   if (PyDict_GetItemString(dict, name))
     return 0;
 
@@ -295,12 +332,16 @@ PyObject* igraphmodule_get_edge_attribute_values(const igraph_t* graph,
 PyObject* igraphmodule_create_or_get_edge_attribute_values(const igraph_t* graph,
     const char* name) {
   PyObject *dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE], *result;
-  if (dict == 0)
+  if (dict == NULL) {
     return 0;
+  }
+
   result = PyDict_GetItemString(dict, name);
-  if (result != 0)
+  if (result != NULL) {
     return result;
-  return igraphmodule_create_edge_attribute(graph, name);
+  }
+  
+  return igraphmodule_i_create_edge_attribute(graph, name);
 }
 
 /* Attribute handlers for the Python interface */
@@ -314,28 +355,37 @@ static igraph_error_t igraphmodule_i_attribute_init(igraph_t *graph, igraph_vect
   if (!attrs) {
     IGRAPH_ERROR("not enough memory to allocate attribute hashes", IGRAPH_ENOMEM);
   }
+  IGRAPH_FINALLY(free, attrs);
+
   if (igraphmodule_i_attribute_struct_init(attrs)) {
     PyErr_PrintEx(0);
-    free(attrs);
     IGRAPH_ERROR("not enough memory to allocate attribute hashes", IGRAPH_ENOMEM);
   }
-  graph->attr = (void*)attrs;
+  IGRAPH_FINALLY(igraphmodule_i_attribute_struct_destroy, attrs);
 
   /* See if we have graph attributes */
   if (attr) {
     PyObject *dict = attrs->attrs[0], *value;
     const char *s;
+  
     n = igraph_vector_ptr_size(attr);
     for (i = 0; i < n; i++) {
       igraph_attribute_record_t *attr_rec;
       attr_rec = VECTOR(*attr)[i];
+
       switch (attr_rec->type) {
       case IGRAPH_ATTRIBUTE_NUMERIC:
         value = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[0]);
+        if (!value) {
+          PyErr_PrintEx(0);
+        }
         break;
       case IGRAPH_ATTRIBUTE_STRING:
         s = igraph_strvector_get((igraph_strvector_t*)attr_rec->value, 0);
         value = PyUnicode_FromString(s ? s : "");
+        if (!value) {
+          PyErr_PrintEx(0);
+        }
         break;
       case IGRAPH_ATTRIBUTE_BOOLEAN:
         value = VECTOR(*(igraph_vector_bool_t*)attr_rec->value)[0] ? Py_True : Py_False;
@@ -343,23 +393,29 @@ static igraph_error_t igraphmodule_i_attribute_init(igraph_t *graph, igraph_vect
         break;
       default:
         IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
-        value = 0;
+        value = NULL;
         break;
       }
+
       if (value) {
         if (PyDict_SetItemString(dict, attr_rec->name, value)) {
           Py_DECREF(value);
-          igraphmodule_i_attribute_struct_destroy(attrs);
-          free(graph->attr); graph->attr = 0;
-          IGRAPH_ERROR("failed to add attributes to graph attribute hash",
-                       IGRAPH_FAILURE);
+          value = NULL;  /* set value to NULL to indicate an error */
+        } else {
+          Py_DECREF(value);
         }
-        Py_DECREF(value);
-        value = 0;
+      }
+      
+      if (!value) {
+        /* there was an error above, bail out */
+        IGRAPH_ERROR("failed to add attributes to graph attribute hash", IGRAPH_FAILURE);
       }
     }
   }
 
+  graph->attr = (void*)attrs;
+  IGRAPH_FINALLY_CLEAN(2);
+
   return IGRAPH_SUCCESS;
 }
 
@@ -370,6 +426,7 @@ static void igraphmodule_i_attribute_destroy(igraph_t *graph) {
   /* printf("Destroying attribute table\n"); */
   if (graph->attr) {
     attrs = (igraphmodule_i_attribute_struct*)graph->attr;
+    graph->attr = NULL;
     igraphmodule_i_attribute_struct_destroy(attrs);
     free(attrs);
   }
@@ -382,50 +439,78 @@ static igraph_error_t igraphmodule_i_attribute_copy(igraph_t *to, const igraph_t
   PyObject *key, *value, *newval, *o=NULL;
   igraph_bool_t copy_attrs[3] = { ga, va, ea };
   int i;
-  Py_ssize_t j, pos = 0;
+  Py_ssize_t j, pos = 0, list_len;
  
   if (from->attr) {
-    fromattrs=ATTR_STRUCT(from);
-    /* what to do with the original value of toattrs? */
-    toattrs=(igraphmodule_i_attribute_struct*)calloc(1, sizeof(igraphmodule_i_attribute_struct));
-    if (!toattrs)
-      IGRAPH_ERROR("not enough memory to allocate attribute hashes", IGRAPH_ENOMEM);
-    if (igraphmodule_i_attribute_struct_init(toattrs)) {
-      PyErr_PrintEx(0);
-      free(toattrs);
+    fromattrs = ATTR_STRUCT(from);
+
+    toattrs = (igraphmodule_i_attribute_struct*) calloc(1, sizeof(igraphmodule_i_attribute_struct));
+    if (!toattrs) {
       IGRAPH_ERROR("not enough memory to allocate attribute hashes", IGRAPH_ENOMEM);
     }
-    to->attr=toattrs;
+    IGRAPH_FINALLY(free, toattrs);
+
+    if (igraphmodule_i_attribute_struct_init(toattrs)) {
+      PyErr_PrintEx(0);
+      IGRAPH_ERROR("not enough memory to allocate attribute hashes", IGRAPH_ENOMEM);
+    }
+    IGRAPH_FINALLY(igraphmodule_i_attribute_struct_destroy, toattrs);
 
     for (i = 0; i < 3; i++) {
-      if (!copy_attrs[i])
-        continue;
-
-      if (!PyDict_Check(fromattrs->attrs[i])) {
-        toattrs->attrs[i]=fromattrs->attrs[i];
-        Py_XINCREF(fromattrs->attrs[i]);
+      if (!copy_attrs[i]) {
         continue;
       }
-      
-      pos = 0;
-      while (PyDict_Next(fromattrs->attrs[i], &pos, &key, &value)) {
-        /* value is only borrowed, so copy it */
-        if (i > 0) {
-          newval = PyList_New(PyList_Size(value));
-          for (j = 0; j < PyList_Size(value); j++) {
+
+      if (!PyDict_Check(fromattrs->attrs[i])) {
+        IGRAPH_ERRORF("expected dict in attribute hash at index %d", IGRAPH_EINVAL, i);
+      }
+
+      /* graph attributes are easy to copy because the dict is a key-value map */
+      if (i == ATTRHASH_IDX_GRAPH) {
+        Py_XDECREF(toattrs->attrs[i]); /* we already had a pre-constructed dict there */
+        toattrs->attrs[i] = PyDict_Copy(fromattrs->attrs[i]);
+        if (!toattrs->attrs[i]) {
+          PyErr_PrintEx(0);
+          IGRAPH_ERROR("cannot copy attribute hashes", IGRAPH_FAILURE);
+        }
+      } else {
+        /* vertex and edge attributes have to be copied in a way that values
+         * are also copied */
+        pos = 0;
+        while (PyDict_Next(fromattrs->attrs[i], &pos, &key, &value)) {
+          /* value is only borrowed, so copy it */
+          if (!PyList_Check(value)) {
+            IGRAPH_ERRORF("expected list in attribute hash at index %d", IGRAPH_EINVAL, i);
+          }
+
+          list_len = PyList_Size(value);
+          newval = PyList_New(list_len);
+          for (j = 0; j < list_len; j++) {
             o = PyList_GetItem(value, j);
             Py_INCREF(o);
             PyList_SetItem(newval, j, o);
           }
-        } else {
-          newval = value;
-          Py_INCREF(newval);
+
+          if (!newval) {
+            PyErr_PrintEx(0);
+            IGRAPH_ERROR("cannot copy attribute hashes", IGRAPH_FAILURE);
+          }
+
+          if (PyDict_SetItem(toattrs->attrs[i], key, newval)) {
+            PyErr_PrintEx(0);
+            Py_DECREF(newval);
+            IGRAPH_ERROR("cannot copy attribute hashes", IGRAPH_FAILURE);
+          }
+
+          Py_DECREF(newval); /* compensate for PyDict_SetItem */
         }
-        PyDict_SetItem(toattrs->attrs[i], key, newval);
-        Py_DECREF(newval); /* compensate for PyDict_SetItem */
       }
     }
+
+    to->attr = toattrs;
+    IGRAPH_FINALLY_CLEAN(2);
   }
+
   return IGRAPH_SUCCESS;
 }
 
@@ -433,9 +518,9 @@ static igraph_error_t igraphmodule_i_attribute_copy(igraph_t *to, const igraph_t
 static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igraph_integer_t nv, igraph_vector_ptr_t *attr) {
   /* Extend the end of every value in the vertex hash with nv pieces of None */
   PyObject *key, *value, *dict;
-  igraph_integer_t i, j, k, l;
+  igraph_integer_t i, j, k, num_attr_entries;
   igraph_attribute_record_t *attr_rec;
-  igraph_bool_t *added_attrs=0;
+  igraph_vector_bool_t added_attrs;
   Py_ssize_t pos = 0;
 
   if (!graph->attr) {
@@ -446,14 +531,8 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
     return IGRAPH_SUCCESS;
   }
 
-  if (attr) {
-    added_attrs = (igraph_bool_t*)calloc((size_t)igraph_vector_ptr_size(attr),
-                                         sizeof(igraph_bool_t));
-    if (!added_attrs) {
-      IGRAPH_ERROR("can't add vertex attributes", IGRAPH_ENOMEM);
-    }
-    IGRAPH_FINALLY(free, added_attrs);
-  }
+  num_attr_entries = attr ? igraph_vector_ptr_size(attr) : 0;
+  IGRAPH_VECTOR_BOOL_INIT_FINALLY(&added_attrs, num_attr_entries);
 
   dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_VERTEX];
   if (!PyDict_Check(dict)) {
@@ -466,17 +545,14 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
     }
 
     /* Check if we have specific values for the given attribute */
-    attr_rec = 0;
-    if (attr) {
-      j = igraph_vector_ptr_size(attr);
-      for (i = 0; i < j; i++) {
-        attr_rec = VECTOR(*attr)[i];
-        if (igraphmodule_PyObject_matches_attribute_record(key, attr_rec)) {
-          added_attrs[i] = 1;
-          break;
-        }
-        attr_rec = 0;
+    attr_rec = NULL;
+    for (i = 0; i < num_attr_entries; i++) {
+      attr_rec = VECTOR(*attr)[i];
+      if (igraphmodule_PyObject_matches_attribute_record(key, attr_rec)) {
+        VECTOR(added_attrs)[i] = 1;
+        break;
       }
+      attr_rec = NULL;
     }
 
     /* If we have specific values for the given attribute, attr_rec contains
@@ -484,7 +560,8 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
     if (attr_rec) {
       for (i = 0; i < nv; i++) {
         const char *s;
-        PyObject *o;
+        PyObject *o = NULL;
+
         switch (attr_rec->type) {
         case IGRAPH_ATTRIBUTE_NUMERIC:
           o = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[i]);
@@ -499,16 +576,24 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
           break;
         default:
           IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
-          o = 0;
+          o = Py_None;
+          Py_INCREF(o);
           break;
         }
+
         if (o) {
-          if (PyList_Append(value, o) == -1) {
-            IGRAPH_ERROR("can't extend a vertex attribute hash member", IGRAPH_FAILURE);
+          if (PyList_Append(value, o)) {
+            Py_DECREF(o); /* append failed */
+            o = NULL; /* indicate error */
           } else {
-            Py_DECREF(o);
+            Py_DECREF(o); /* drop reference, the list has it now */
           }
         }
+
+        if (!o) {
+          PyErr_PrintEx(0);
+          IGRAPH_ERROR("can't extend a vertex attribute hash member", IGRAPH_FAILURE);
+        }
       }
 
       /* Invalidate the vertex name index if needed */
@@ -517,7 +602,8 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
       }
     } else {
       for (i = 0; i < nv; i++) {
-        if (PyList_Append(value, Py_None) == -1) {
+        if (PyList_Append(value, Py_None)) {
+          PyErr_PrintEx(0);
           IGRAPH_ERROR("can't extend a vertex attribute hash member", IGRAPH_FAILURE);
         }
       }
@@ -526,64 +612,75 @@ static igraph_error_t igraphmodule_i_attribute_add_vertices(igraph_t *graph, igr
 
   /* Okay, now we added the new attribute values for the already existing
    * attribute keys. Let's see if we have something left */
-  if (attr) {
-    l = igraph_vector_ptr_size(attr);
-    j = igraph_vcount(graph) - nv;
-    /* j contains the number of vertices EXCLUDING the new ones! */
-    for (k = 0; k < l; k++) {
-      if (added_attrs[k]) {
-        continue;
-      }
-      attr_rec = (igraph_attribute_record_t*)VECTOR(*attr)[k];
+  j = igraph_vcount(graph) - nv;
 
-      value = PyList_New(j + nv);
-      if (!value) {
-        IGRAPH_ERROR("can't add attributes", IGRAPH_ENOMEM);
-      }
-
-      for (i = 0; i < j; i++) {
-        Py_INCREF(Py_None);
-        PyList_SetItem(value, i, Py_None);
-      }
-
-      for (i = 0; i < nv; i++) {
-        const char *s;
-        PyObject *o;
-        switch (attr_rec->type) {
-        case IGRAPH_ATTRIBUTE_NUMERIC:
-          o = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[i]);
-          break;
-        case IGRAPH_ATTRIBUTE_STRING:
-          s = igraph_strvector_get((igraph_strvector_t*)attr_rec->value, i);
-          o = PyUnicode_FromString(s);
-          break;
-        case IGRAPH_ATTRIBUTE_BOOLEAN:
-          o = VECTOR(*(igraph_vector_bool_t*)attr_rec->value)[i] ? Py_True : Py_False;
-          Py_INCREF(o);
-          break;
-        default:
-          IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
-          o = 0;
-          break;
-        }
-        if (o) {
-          PyList_SetItem(value, i + j, o);
-        }
-      }
-
-      /* Invalidate the vertex name index if needed */
-      if (!strcmp(attr_rec->name, "name")) {
-        igraphmodule_i_attribute_struct_invalidate_vertex_name_index(ATTR_STRUCT(graph));
-      }
-
-      PyDict_SetItemString(dict, attr_rec->name, value);
-      Py_DECREF(value);   /* compensate for PyDict_SetItemString */
+  /* j contains the number of vertices EXCLUDING the new ones! */
+  for (k = 0; k < num_attr_entries; k++) {
+    if (VECTOR(added_attrs)[k]) {
+      continue;
     }
 
-    free(added_attrs);
-    IGRAPH_FINALLY_CLEAN(1);
+    attr_rec = (igraph_attribute_record_t*)VECTOR(*attr)[k];
+
+    value = PyList_New(j + nv);
+    if (!value) {
+      IGRAPH_ERROR("can't add attributes", IGRAPH_ENOMEM);
+    }
+
+    for (i = 0; i < j; i++) {
+      Py_INCREF(Py_None);
+      PyList_SetItem(value, i, Py_None);
+    }
+
+    for (i = 0; i < nv; i++) {
+      const char *s;
+      PyObject *o = NULL;
+      switch (attr_rec->type) {
+      case IGRAPH_ATTRIBUTE_NUMERIC:
+        o = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[i]);
+        break;
+      case IGRAPH_ATTRIBUTE_STRING:
+        s = igraph_strvector_get((igraph_strvector_t*)attr_rec->value, i);
+        o = PyUnicode_FromString(s);
+        break;
+      case IGRAPH_ATTRIBUTE_BOOLEAN:
+        o = VECTOR(*(igraph_vector_bool_t*)attr_rec->value)[i] ? Py_True : Py_False;
+        Py_INCREF(o);
+        break;
+      default:
+        IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
+        o = Py_None;
+        Py_INCREF(o);
+        break;
+      }
+
+      if (o) {
+        if (PyList_SetItem(value, i + j, o)) {  
+          Py_DECREF(o); /* append failed */
+          o = NULL; /* indicate error */
+        } else {
+          /* reference stolen by the list */
+        }
+      }
+
+      if (!o) {
+        PyErr_PrintEx(0);
+        IGRAPH_ERROR("can't extend a vertex attribute hash member", IGRAPH_FAILURE);
+      }
+    }
+
+    /* Invalidate the vertex name index if needed */
+    if (!strcmp(attr_rec->name, "name")) {
+      igraphmodule_i_attribute_struct_invalidate_vertex_name_index(ATTR_STRUCT(graph));
+    }
+
+    PyDict_SetItemString(dict, attr_rec->name, value);
+    Py_DECREF(value);   /* compensate for PyDict_SetItemString */
   }
 
+  igraph_vector_bool_destroy(&added_attrs);
+  IGRAPH_FINALLY_CLEAN(1);
+
   return IGRAPH_SUCCESS;
 }
 
@@ -596,12 +693,12 @@ static igraph_error_t igraphmodule_i_attribute_permute_vertices(const igraph_t *
   
   dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_VERTEX];
   if (!PyDict_Check(dict)) {
-    return 1;
+    IGRAPH_ERROR("vertex attribute hash type mismatch", IGRAPH_EINVAL);
   }
 
   newdict = PyDict_New();
   if (!newdict) {
-    return 1;
+    IGRAPH_ERROR("cannot allocate new dict for vertex permutation", IGRAPH_ENOMEM);
   }
 
   n = igraph_vector_int_size(idx);
@@ -612,17 +709,19 @@ static igraph_error_t igraphmodule_i_attribute_permute_vertices(const igraph_t *
     for (i = 0; i < n; i++) {
       o = PyList_GetItem(value, VECTOR(*idx)[i]);
       if (!o) {
+        PyErr_PrintEx(0);
         Py_DECREF(newlist);
         Py_DECREF(newdict);
         PyErr_Clear();
-        return 1;
+        IGRAPH_ERROR("", IGRAPH_FAILURE);
       }
       Py_INCREF(o);
       if (PyList_SetItem(newlist, i, o)) {
+        PyErr_PrintEx(0);
         Py_DECREF(o);
         Py_DECREF(newlist);
         Py_DECREF(newdict);
-        return 1;
+        IGRAPH_ERROR("", IGRAPH_FAILURE);
       }
     }
     PyDict_SetItem(newdict, key, newlist);
@@ -636,7 +735,7 @@ static igraph_error_t igraphmodule_i_attribute_permute_vertices(const igraph_t *
   /* Invalidate the vertex name index */
   igraphmodule_i_attribute_struct_invalidate_vertex_name_index(ATTR_STRUCT(newgraph));
 
-  return 0;
+  return IGRAPH_SUCCESS;
 }
 
 /* Adding edges */
@@ -644,8 +743,8 @@ static igraph_error_t igraphmodule_i_attribute_add_edges(igraph_t *graph, const
   /* Extend the end of every value in the edge hash with ne pieces of None */
   PyObject *key, *value, *dict;
   Py_ssize_t pos = 0;
-  igraph_integer_t i, j, k, l, ne;
-  igraph_bool_t *added_attrs=0;
+  igraph_integer_t i, j, k, ne, num_attr_entries;
+  igraph_vector_bool_t added_attrs;
   igraph_attribute_record_t *attr_rec;
 
   if (!graph->attr) {
@@ -657,14 +756,8 @@ static igraph_error_t igraphmodule_i_attribute_add_edges(igraph_t *graph, const
     return IGRAPH_SUCCESS;
   }
 
-  if (attr) {
-    added_attrs = (igraph_bool_t*)calloc((size_t)igraph_vector_ptr_size(attr),
-                                         sizeof(igraph_bool_t));
-    if (!added_attrs) {
-      IGRAPH_ERROR("can't add vertex attributes", IGRAPH_ENOMEM);
-    }
-    IGRAPH_FINALLY(free, added_attrs);
-  }
+  num_attr_entries = attr ? igraph_vector_ptr_size(attr) : 0;
+  IGRAPH_VECTOR_BOOL_INIT_FINALLY(&added_attrs, num_attr_entries);
 
   dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE];
   if (!PyDict_Check(dict)) {
@@ -677,18 +770,16 @@ static igraph_error_t igraphmodule_i_attribute_add_edges(igraph_t *graph, const
     }
 
     /* Check if we have specific values for the given attribute */
-    attr_rec = 0;
-    if (attr) {
-      j = igraph_vector_ptr_size(attr);
-      for (i = 0; i < j; i++) {
-        attr_rec = VECTOR(*attr)[i];
-        if (igraphmodule_PyObject_matches_attribute_record(key, attr_rec)) {
-          added_attrs[i] = 1;
-          break;
-        }
-        attr_rec = 0;
+    attr_rec = NULL;
+    for (i = 0; i < num_attr_entries; i++) {
+      attr_rec = VECTOR(*attr)[i];
+      if (igraphmodule_PyObject_matches_attribute_record(key, attr_rec)) {
+        VECTOR(added_attrs)[i] = 1;
+        break;
       }
+      attr_rec = NULL;
     }
+
     /* If we have specific values for the given attribute, attr_rec contains
      * the appropriate vector. If not, it is null. */
     if (attr_rec) {
@@ -709,86 +800,99 @@ static igraph_error_t igraphmodule_i_attribute_add_edges(igraph_t *graph, const
           break;
         default:
           IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
-          o = 0;
+          o = Py_None;
+          Py_INCREF(o);
           break;
         }
+
         if (o) {
-          if (PyList_Append(value, o) == -1) {
-            IGRAPH_ERROR("can't extend an edge attribute hash member", IGRAPH_FAILURE);
+          if (PyList_Append(value, o)) {
+            Py_DECREF(o); /* append failed */
+            o = NULL; /* indicate error */
           } else {
-            Py_DECREF(o);
+            Py_DECREF(o); /* drop reference, the list has it now */
           }
         }
+
+        if (!o) {
+          PyErr_PrintEx(0);
+          IGRAPH_ERROR("can't extend an edge attribute hash member", IGRAPH_FAILURE);
+        }
       }
     } else {
       for (i = 0; i < ne; i++) {
-        if (PyList_Append(value, Py_None) == -1) {
+        if (PyList_Append(value, Py_None)) {
+          PyErr_PrintEx(0);
           IGRAPH_ERROR("can't extend an edge attribute hash member", IGRAPH_FAILURE);
         }
       }
     }
   }
   
-  /*pos=0;
-  while (PyDict_Next(dict, &pos, &key, &value)) {
-    printf("key: "); PyObject_Print(key, stdout, Py_PRINT_RAW); printf("\n");
-    printf("value: "); PyObject_Print(value, stdout, Py_PRINT_RAW); printf("\n");
-  }*/
-  
   /* Okay, now we added the new attribute values for the already existing
    * attribute keys. Let's see if we have something left */
-  if (attr) {
-    l = igraph_vector_ptr_size(attr);
-    j = igraph_ecount(graph) - ne;
-    /* j contains the number of edges EXCLUDING the new ones! */
-    for (k = 0; k < l; k++) {
-      if (added_attrs[k]) {
-        continue;
-      }
-      attr_rec=(igraph_attribute_record_t*)VECTOR(*attr)[k];
-
-      value = PyList_New(j + ne);
-      if (!value) {
-        IGRAPH_ERROR("can't add attributes", IGRAPH_ENOMEM);
-      }
-
-      for (i = 0; i < j; i++) {
-        Py_INCREF(Py_None);
-        PyList_SetItem(value, i, Py_None);
-      }
-
-      for (i = 0; i < ne; i++) {
-        const char *s;
-        PyObject *o;
-        switch (attr_rec->type) {
-        case IGRAPH_ATTRIBUTE_NUMERIC:
-          o = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[i]);
-          break;
-        case IGRAPH_ATTRIBUTE_STRING:
-          s = igraph_strvector_get((igraph_strvector_t*)attr_rec->value, i);
-          o = PyUnicode_FromString(s);
-          break;
-        case IGRAPH_ATTRIBUTE_BOOLEAN:
-          o = VECTOR(*(igraph_vector_bool_t*)attr_rec->value)[i] ? Py_True : Py_False;
-          Py_INCREF(o);
-          break;
-        default:
-          IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
-          o = 0;
-          break;
-        }
-        if (o) {
-          PyList_SetItem(value, i + j, o);
-        }
-      }
-
-      PyDict_SetItemString(dict, attr_rec->name, value);
-      Py_DECREF(value);   /* compensate for PyDict_SetItemString */
+  j = igraph_ecount(graph) - ne;
+  /* j contains the number of edges EXCLUDING the new ones! */
+  for (k = 0; k < num_attr_entries; k++) {
+    if (VECTOR(added_attrs)[k]) {
+      continue;
     }
-    free(added_attrs);
-    IGRAPH_FINALLY_CLEAN(1);
+    attr_rec=(igraph_attribute_record_t*)VECTOR(*attr)[k];
+
+    value = PyList_New(j + ne);
+    if (!value) {
+      IGRAPH_ERROR("can't add attributes", IGRAPH_ENOMEM);
+    }
+
+    for (i = 0; i < j; i++) {
+      Py_INCREF(Py_None);
+      PyList_SetItem(value, i, Py_None);
+    }
+
+    for (i = 0; i < ne; i++) {
+      const char *s;
+      PyObject *o;
+      switch (attr_rec->type) {
+      case IGRAPH_ATTRIBUTE_NUMERIC:
+        o = PyFloat_FromDouble((double)VECTOR(*(igraph_vector_t*)attr_rec->value)[i]);
+        break;
+      case IGRAPH_ATTRIBUTE_STRING:
+        s = igraph_strvector_get((igraph_strvector_t*)attr_rec->value, i);
+        o = PyUnicode_FromString(s);
+        break;
+      case IGRAPH_ATTRIBUTE_BOOLEAN:
+        o = VECTOR(*(igraph_vector_bool_t*)attr_rec->value)[i] ? Py_True : Py_False;
+        Py_INCREF(o);
+        break;
+      default:
+        IGRAPH_WARNING("unsupported attribute type (not string, numeric or Boolean)");
+        o = Py_None;
+        Py_INCREF(o);
+        break;
+      }
+
+      if (o) {
+        if (PyList_SetItem(value, i + j, o)) {  
+          Py_DECREF(o); /* append failed */
+          o = NULL; /* indicate error */
+        } else {
+          /* reference stolen by the list */
+        }
+      }
+
+      if (!o) {
+        PyErr_PrintEx(0);
+        IGRAPH_ERROR("can't extend a vertex attribute hash member", IGRAPH_FAILURE);
+      }
+    }
+
+    PyDict_SetItemString(dict, attr_rec->name, value);
+    Py_DECREF(value);   /* compensate for PyDict_SetItemString */
   }
 
+  igraph_vector_bool_destroy(&added_attrs);
+  IGRAPH_FINALLY_CLEAN(1);
+
   return IGRAPH_SUCCESS;
 }
 
@@ -801,12 +905,12 @@ static igraph_error_t igraphmodule_i_attribute_permute_edges(const igraph_t *gra
 
   dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE];
   if (!PyDict_Check(dict)) {
-    return 1;
+    IGRAPH_ERROR("edge attribute hash type mismatch", IGRAPH_EINVAL);
   }
 
   newdict = PyDict_New();
   if (!newdict) {
-    return 1;
+    IGRAPH_ERROR("cannot allocate new dict for edge permutation", IGRAPH_ENOMEM);
   }
 
   n = igraph_vector_int_size(idx);
@@ -815,19 +919,21 @@ static igraph_error_t igraphmodule_i_attribute_permute_edges(const igraph_t *gra
   while (PyDict_Next(dict, &pos, &key, &value)) {
     newlist = PyList_New(n);
     for (i = 0; i < n; i++) {
-      o=PyList_GetItem(value, VECTOR(*idx)[i]);
+      o = PyList_GetItem(value, VECTOR(*idx)[i]);
       if (!o) {
+        PyErr_PrintEx(0);
         Py_DECREF(newlist);
         Py_DECREF(newdict);
         PyErr_Clear();
-        return 1;
+        IGRAPH_ERROR("", IGRAPH_FAILURE);
       }
       Py_INCREF(o);
       if (PyList_SetItem(newlist, i, o)) {
+        PyErr_PrintEx(0);
         Py_DECREF(o);
         Py_DECREF(newlist);
         Py_DECREF(newdict);
-        return 1;
+        IGRAPH_ERROR("", IGRAPH_FAILURE);
       }
     }
     PyDict_SetItem(newdict, key, newlist);
@@ -838,7 +944,7 @@ static igraph_error_t igraphmodule_i_attribute_permute_edges(const igraph_t *gra
   ATTR_STRUCT_DICT(newgraph)[ATTRHASH_IDX_EDGE]=newdict;
   Py_DECREF(dict);
 
-  return 0;
+  return IGRAPH_SUCCESS;
 }
 
 /* Auxiliary function for combining vertices/edges. Given a merge list
@@ -1311,23 +1417,24 @@ static igraph_error_t igraphmodule_i_attribute_combine_dicts(PyObject *dict,
   /* Allocate memory for the attribute_combination_records */
   n = PyDict_Size(dict);
   todo = (igraph_attribute_combination_record_t*)calloc(
-    n+1, sizeof(igraph_attribute_combination_record_t)
+    n + 1, sizeof(igraph_attribute_combination_record_t)
   );
   if (todo == 0) {
     IGRAPH_ERROR("cannot allocate memory for attribute combination", IGRAPH_ENOMEM);
   }
-  for (i = 0; i < n+1; i++)
+  for (i = 0; i < n + 1; i++) {
     todo[i].name = 0;       /* sentinel elements */
+  }
   IGRAPH_FINALLY(igraphmodule_i_free_attribute_combination_records, todo);
 
   /* Collect what to do for each attribute in the source dict */
   pos = 0; i = 0;
   while (PyDict_Next(dict, &pos, &key, &value)) {
     todo[i].name = PyUnicode_CopyAsString(key);
-    if (todo[i].name == 0)
+    if (todo[i].name == 0) {
       IGRAPH_ERROR("PyUnicode_CopyAsString failed", IGRAPH_FAILURE);
-    igraph_attribute_combination_query(comb, todo[i].name,
-        &todo[i].type, &todo[i].func);
+    }
+    IGRAPH_CHECK(igraph_attribute_combination_query(comb, todo[i].name, &todo[i].type, &todo[i].func));
     i++;
   }
 
@@ -1458,8 +1565,8 @@ static igraph_error_t igraphmodule_i_attribute_combine_edges(const igraph_t *gra
   PyObject *dict, *newdict;
 
   /* Get the attribute dicts */
-  dict=ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE];
-  newdict=ATTR_STRUCT_DICT(newgraph)[ATTRHASH_IDX_EDGE];
+  dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_EDGE];
+  newdict = ATTR_STRUCT_DICT(newgraph)[ATTRHASH_IDX_EDGE];
 
   return igraphmodule_i_attribute_combine_dicts(dict, newdict,
       merges, comb);
@@ -1493,13 +1600,13 @@ static igraph_error_t igraphmodule_i_attribute_get_info(const igraph_t *graph,
     if (n) {
       retval = igraphmodule_PyList_to_existing_strvector_t(keys, n);
       if (retval) {
-        return retval ? IGRAPH_FAILURE : IGRAPH_SUCCESS;
+        IGRAPH_ERROR("Cannot convert Python list to existing igraph_strvector_t", IGRAPH_FAILURE);
       }
     }
 
     if (t) {
       k = PyList_Size(keys);
-      igraph_vector_int_resize(t, k);
+      IGRAPH_CHECK(igraph_vector_int_resize(t, k));
       for (j = 0; j < k; j++) {
         int is_numeric = 1;
         int is_string = 1;
@@ -1594,7 +1701,7 @@ igraph_error_t igraphmodule_i_attribute_get_type(const igraph_t *graph,
   }
 
   /* Basic type check */
-  if (!PyList_Check(o)) {
+  if (attrnum != ATTRHASH_IDX_GRAPH && !PyList_Check(o)) {
     IGRAPH_ERROR("attribute hash type mismatch", IGRAPH_EINVAL);
   }
 
@@ -1657,8 +1764,9 @@ igraph_error_t igraphmodule_i_get_boolean_graph_attr(const igraph_t *graph,
   /* No error checking, if we get here, the type has already been checked by previous
      attribute handler calls... hopefully :) Same applies for the other handlers. */
   o = PyDict_GetItemString(dict, name);
-  if (!o)
+  if (!o) {
     IGRAPH_ERROR("No such attribute", IGRAPH_EINVAL);
+  }
   IGRAPH_CHECK(igraph_vector_bool_resize(value, 1));
   VECTOR(*value)[0] = PyObject_IsTrue(o);
   return IGRAPH_SUCCESS;
@@ -1672,7 +1780,9 @@ igraph_error_t igraphmodule_i_get_numeric_graph_attr(const igraph_t *graph,
   /* No error checking, if we get here, the type has already been checked by previous
      attribute handler calls... hopefully :) Same applies for the other handlers. */
   o = PyDict_GetItemString(dict, name);
-  if (!o) IGRAPH_ERROR("No such attribute", IGRAPH_EINVAL);
+  if (!o) {
+    IGRAPH_ERROR("No such attribute", IGRAPH_EINVAL);
+  }
   IGRAPH_CHECK(igraph_vector_resize(value, 1));
   if (o == Py_None) {
     VECTOR(*value)[0] = IGRAPH_NAN;
@@ -1695,8 +1805,9 @@ igraph_error_t igraphmodule_i_get_string_graph_attr(const igraph_t *graph,
 
   dict = ATTR_STRUCT_DICT(graph)[ATTRHASH_IDX_GRAPH];
   o = PyDict_GetItemString(dict, name);
-  if (!o)
+  if (!o) {
     IGRAPH_ERROR("No such attribute", IGRAPH_EINVAL);
+  }
   IGRAPH_CHECK(igraph_strvector_resize(value, 1));
 
   /* For Python 3.x, we simply call PyObject_Str, which produces a
@@ -1709,8 +1820,9 @@ igraph_error_t igraphmodule_i_get_string_graph_attr(const igraph_t *graph,
     Py_INCREF(str);
   } else {
     PyObject* unicode = PyObject_Str(o);
-    if (unicode == 0)
+    if (unicode == 0) {
       IGRAPH_ERROR("Internal error in PyObject_Str", IGRAPH_EINVAL);
+    }
     str = PyUnicode_AsEncodedString(unicode, "utf-8", "xmlcharrefreplace");
     Py_DECREF(unicode);
   }
